@@ -1,7 +1,7 @@
 
 /*
  * Copyright (c) 2009, Sun Microsystems, Inc.
- * Copyright (c) 2012-2017 Red Hat, Inc. and/or its affiliates.
+ * Copyright (c) 2012-2018 Red Hat, Inc. and/or its affiliates.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -32,7 +32,7 @@
  * Copyright (c) 1986-1991 by Sun Microsystems Inc.
  */
 
-#include <config.h>
+#include "config.h"
 
 /*
  * svc_dg.c, Server side for connectionless RPC.
@@ -94,6 +94,8 @@ svc_dg_xprt_zalloc(size_t iosz)
 
 	/* Init SVCXPRT locks, etc */
 	rpc_dplx_rec_init(&su->su_dr);
+	/* Extra ref to match TCP */
+	SVC_REF(&su->su_dr.xprt, SVC_REF_FLAG_NONE);
 	xdr_ioq_setup(&su->su_dr.ioq);
 	return (su);
 }
@@ -199,7 +201,7 @@ svc_dg_ncreatef(const int fd, const u_int sendsz, const u_int recvsz,
 	     && !(flags & SVC_CREATE_FLAG_XPRT_NOREG))
 	    || (flags & SVC_CREATE_FLAG_XPRT_DOREG))
 		svc_rqst_evchan_reg(__svc_params->ev_u.evchan.id, xprt,
-				    SVC_RQST_FLAG_LOCKED |
+				    RPC_DPLX_LOCKED |
 				    SVC_RQST_FLAG_CHAN_AFFINITY);
 
 	/* release */
@@ -305,14 +307,22 @@ svc_dg_rendezvous(SVCXPRT *xprt)
 static enum xprt_stat
 svc_dg_recv(SVCXPRT *xprt)
 {
-	/* Stolen reference! Callee must take a reference to rq_xprt. */
-	xprt->xp_refs--;
-	XPRT_TRACE(xprt, __func__, __func__, __LINE__);
+	enum xprt_stat stat;
 
 	/* pass the xdrs to user to store in struct svc_req, as most of
 	 * the work has already been done on rendezvous
 	 */
-	return (__svc_params->request_cb(xprt, REC_XPRT(xprt)->ioq.xdrs));
+	stat = __svc_params->request_cb(xprt, REC_XPRT(xprt)->ioq.xdrs);
+
+	if (xprt->xp_flags & SVC_XPRT_FLAG_DESTROYED)
+		return (XPRT_DESTROYED);
+
+	/* Only after checking SVC_XPRT_FLAG_DESTROYED:
+	 * because SVC_DESTROY() has decremented already.
+	 */
+	SVC_DESTROY(xprt);
+	SVC_RELEASE(xprt, SVC_RELEASE_FLAG_NONE);
+	return (stat);
 }
 
 static enum xprt_stat
@@ -423,13 +433,14 @@ svc_dg_destroy_task(struct work_pool_entry *wpe)
 {
 	struct rpc_dplx_rec *rec =
 			opr_containerof(wpe, struct rpc_dplx_rec, ioq.ioq_wpe);
+	SVCXPRT *xprt = &rec->xprt;
 	uint16_t xp_flags;
 
 	__warnx(TIRPC_DEBUG_FLAG_REFCNT,
-		"%s() %p fd %d xp_refs %" PRIu32,
-		__func__, rec, rec->xprt.xp_fd, rec->xprt.xp_refs);
+		"%s() %p fd %d xp_refcnt %" PRId32,
+		__func__, xprt, xprt->xp_fd, xprt->xp_refcnt);
 
-	if (rec->xprt.xp_refs) {
+	if (rec->xprt.xp_refcnt) {
 		/* instead of nanosleep */
 		work_pool_submit(&svc_work_pool, &(rec->ioq.ioq_wpe));
 		return;
@@ -467,13 +478,12 @@ svc_dg_destroy_it(SVCXPRT *xprt, u_int flags, const char *tag, const int line)
 
 	if (!xprt->xp_parent) {
 		/* only original parent is registered */
-		svc_rqst_xprt_unregister(xprt);
+		svc_rqst_xprt_unregister(xprt, flags);
 	}
 
 	__warnx(TIRPC_DEBUG_FLAG_REFCNT,
-		"%s() %p fd %d xp_refs %" PRIu32
-		" should actually destroy things @ %s:%d",
-		__func__, xprt, xprt->xp_fd, xprt->xp_refs, tag, line);
+		"%s() %p fd %d xp_refcnt %" PRId32 " @%s:%d",
+		__func__, xprt, xprt->xp_fd, xprt->xp_refcnt, tag, line);
 
 	while (atomic_postset_uint16_t_bits(&(REC_XPRT(xprt)->ioq.ioq_s.qflags),
 					    IOQ_FLAG_WORKING)
